const log = require('j7/log');
const bcutils = require('j7/bcutils');
const utils = require('j7/utils');
const bcconst = require('common/bcconst');
const BaseEventProcess = require('../common/BaseEventProcess');
const metaFactory = require('../../../metadata/factory');

class Transfer extends BaseEventProcess {

  async start() {
    const returnValues = this.getReturnValues();
    const from = returnValues['from'];
    const to = returnValues['to'];
    const tokenId = returnValues['tokenId'];

    if (bcutils.isSysAddress(from)) {
      const airDropMeta = metaFactory.getAirDrop(
        tokenId,
        this.getNetId(),
        bcconst.BC_NFT_HERO,
        this.getContractAddress());
      if (airDropMeta) {
        const exists = await this.exists721Nft(tokenId, this.getContractAddress());
        if (!exists) {
          await this.mint721Nft(
            to,
            tokenId,
            airDropMeta['item_id'],
            bcconst.BC_NFT_HERO,
            this.getContractAddress(),
            this.getBlockNumber()
          );
        }
        await this.apiMint(to, tokenId, airDropMeta);
      }
    }

    await this.add721NftRefresh
    (
      this.getNetId(),
      this.getContractAddress(),
      this.getContractName(),
      tokenId
    );
    await this.update721NftOwner(tokenId, this.getContractAddress(), to);
    await this.markOk();
  }

  async apiMint(address, tokenId, airDropMeta) {
    console.log('apiMint', address, tokenId, airDropMeta);
    const params = {
      'c': 'Callback',
      'a': 'dispatch',
      'action': 'mintNftHero',
      'address': bcutils.toNormalAddress(address),
      'tokenId': tokenId,
      'itemId': airDropMeta['item_id'],
      'quality': airDropMeta['quality'],
    };
    await this.callGameApi
    (params,
     async (data) => {
        if (!data) {
          this.throwError('callGameApi error:' + data);
        }
        if (data['errcode'] == 0 ||
           data['errcode'] == 1) {
          return true;
        }
       return true;
     });
  }

}

module.exports = Transfer;
