const utils = require('j7/utils');
const events = {};

async function internalAdd(clsName, modName) {
  const modClass = require('./' + modName);
  events[clsName] = {
    'clsName': clsName,
    'modName': modName,
    'class': modClass,
  };
  if (modClass.staticInit) {
    await modClass.staticInit();
  }
}

async function add(clsName, modName) {
  await internalAdd(clsName, modName);
}

async function init() {
  await add('HERO.Transfer', 'HERO/transfer');
  await add('CHIP.Transfer', 'CHIP/transfer');
  await add('Gacha.Transfer', 'Gacha/transfer');
  await add('Explorer.Transfer', 'Explorer/transfer');
  await add('Genesis.Transfer', 'Genesis/transfer');
  await add('PLANET.Transfer', 'PLANET/transfer');
  await add('UserMinterFactory.TokenMinted', 'UserMinterFactory/activate721nft');
  await add('BENftMarket.SellOrder', 'BENftMarket/sell_order');
  await add('BENftMarket.CancelOrder', 'BENftMarket/cancel_order');
  await add('BENftMarket.PriceUpdate', 'BENftMarket/price_update');
  await add('BENftMarket.BuyOrder', 'BENftMarket/buy_order');
  await add('GameItemMall.ItemSoldOut', 'GameItemMall/item_soldout');
  await add('GameItemMarket.ItemSoldOut', 'GameItemMarket/item_soldout');
  await add('ERC721Staking.Staked', 'ERC721Staking/staked');
  await add('ERC721Staking.Redeem', 'ERC721Staking/redeem');
}

function getEventHandle(eventProc) {
  const clsName = eventProc.getContractName() + '.' + eventProc.getEventName();
  return utils.hasKey(events, clsName) ? events[clsName] : null;
}

function createEventHandle(eventProc, conn, eventDb) {
  const clsName = eventProc.getContractName() + '.' + eventProc.getEventName();
  return new events[clsName]['class'](eventProc, conn, eventDb);
}

exports.init = init;
exports.getEventHandle = getEventHandle;
exports.createEventHandle = createEventHandle;
