const app = require('j7/app');
const utils = require('j7/utils');
const bcutils = require('j7/bcutils');
const bcconst = require('common/bcconst');
const metaFactory = require('../metadata/factory');
const serviceFactory = require('../services/factory');

async function claim(session) {
  try {
    const account = bcutils.toNormalAddress(session.request('account', ''));
    const netId = session.request('net_id');
    const bc = serviceFactory.create('BlockChain');
    {
      const ret = await bc.init(netId);
      if (!ret) {
        session.rspErr(500, 'net_id error');
        return;
      }
    }
    {
      if (!bc.isValidAddress(account)) {
        session.rspErr(500, 'account error');
        return;
      }
    }

    const nftAddress = bc.getNftAddress(bcconst.BC_NFT_CLAIM_BOX);

    const transModel = session.createModel('Transaction');
    const usedTokenIdModel = session.createModel('UsedTokenId');

    const {err, seqId} = await transModel.add(account, session.requestToJson(), netId);
    if (err) {
      session.rspErr(500, 'server internal error');
      return;
    }

    const nowTime = utils.getUtcTime();
    const tokenId = bcutils.genTokenId(
      2006,
      bcconst.BC_FUNC_COMMON,
      nowTime,
      1,
      seqId
    );
    const transId = bcutils.genTransId
    (
      bcconst.BC_TRANS_FUNC_COMMON,
      nowTime,
      seqId,
      [
        tokenId
      ]
    );
    await transModel.update
    (
      seqId,
      [
        ['trans_id', transId]
      ]);
    const nonce = transId;
    const signature = await bc.soliditySha3Sign(
      account,
      nftAddress,
      nowTime,
      nonce
    );

    const rspParams = [
      nftAddress,
      nowTime,
      nonce,
      signature
    ];
    utils.arrayToStrings(rspParams);

    session.rspData({
      'trans_id' : transId,
      'params': rspParams
    });
  } catch (e) {
    session.rspErr(1, 1);
    utils.safeDumpErrStack(e);
    console.log(e);
  }
}

function init() {
  app.registerHandler('ClaimBox', 'claim', claim);
}

exports.init = init;
